#!/bin/bash

declare -r scriptVersion="1.1"
declare -r acpi_call_path="/proc/acpi/call"
declare -r conservation_mode_path="/sys/bus/platform/drivers/ideapad_acpi/VPC2004:00/conservation_mode"
declare -ri batterySaving=0
declare -ri intelligentCooling=1
declare -ri extremePerformance=2
declare -r acpiReturnValue0="0x0"
declare -r acpiReturnValue1="0x1"
declare -r acpiReturnValue2="0x2"

check_root()
{
	local -i userId=$(id -u)
	if [ $userId -ne 0 ]
	then
		echo "Must be run with root privileges."
		exit 126
	fi
}

print_bios_version()
{
	local bios_version=$(dmidecode -s bios-version)
	echo "Bios version: $bios_version"
}

print_system_version()
{
	dmidecode -s system-version
}

get_system_version()
{
	local dmi_system_version=$(print_system_version)
	if [ "$dmi_system_version" = "IdeaPad 5 15ARE05" ]
	then
		return 15
	elif [ "$dmi_system_version" = "IdeaPad 5 14ARE05" ]
	then 
		return 14
	else
		exit 3
	fi
}

print_power_state()
{
	acpi -ba
}

load_acpi_module()
{
	if [ ! -e $acpi_call_path ]
	then
		echo "Loading acpi_call kernel module."
		modprobe acpi_call > /dev/null 2>&1
	fi
	if [ ! -e $acpi_call_path ]
	then
		echo "Kernel module acpi_call not installed."
		exit 127
	fi
}

get_acpi_return_code()
{
	cat $acpi_call_path | cut -d '' -f1
}

get_conservation_mode()
{
	if [ -r $conservation_mode_path ]
	then
		local retConvMode=$(cat $conservation_mode_path)
		return $retConvMode
	else
		load_acpi_module
		case $system_version in
			14 )	echo '\_SB.PCI0.LPC0.EC0.BTSG' > $acpi_call_path
					;;
			15 )	echo '\_SB.PCI0.LPC0.EC0.BTSM' > $acpi_call_path
					;;
		esac
		local retAcpiConvMode=$(get_acpi_return_code)
		case "$retAcpiConvMode" in 
			$acpiReturnValue0 )	return 0
						;;
			$acpiReturnValue1 )	return 1
			       			;;
			* ) 			echo "Can't read Battery Conservation mode."
						exit 3
		esac	
	fi
}

set_conservation_mode()
{
	local -i arg1=$1
	if [ -w $conservation_mode_path ]
	then
		echo $arg1 > $conservation_mode_path 
	else
		load_acpi_module
		if [ $arg1 -eq 0 ]
		then
			echo '\_SB.PCI0.LPC0.EC0.VPC0.SBMC 0x05' > $acpi_call_path
		else
			echo '\_SB.PCI0.LPC0.EC0.VPC0.SBMC 0x03' > $acpi_call_path
		fi
	fi
}

toggle_conservation_mode()
{
	get_conservation_mode
	local -i retConvMode=$?
	if [ $retConvMode -eq 0 ]
	then
		echo "Toggling Battery Conservation mode: ON"
		set_conservation_mode 1
	else 
		echo "Toggling Battery Conservation mode: OFF"
		set_conservation_mode 0
	fi
}

print_conservation_mode_status()
{
	get_conservation_mode
	local -i retConvMode=$?
	if [ $retConvMode -eq 0 ]
	then
		echo "Battery Conservation mode: OFF"
	elif [ $retConvMode -eq 1 ]
	then
		echo "Battery Conservation mode: ON"
	fi
	
}

get_system_performance_mode_14()
{
	echo "\_SB.PCI0.LPC0.EC0.STMD" > $acpi_call_path
	local retSTMD=$(get_acpi_return_code)
	echo "\_SB.PCI0.LPC0.EC0.QTMD" > $acpi_call_path
	local retQTMD=$(get_acpi_return_code)
	if [ "$retSTMD" = $acpiReturnValue0 ] && [ "$retQTMD" = $acpiReturnValue0 ]
	then
		return $extremePerformance
	elif [ "$retSTMD" = $acpiReturnValue0 ] && [ "$retQTMD" = $acpiReturnValue1 ]
	then
		return $batterySaving
	elif [ "$retSTMD" = $acpiReturnValue1 ] && [ "$retQTMD" = $acpiReturnValue0 ]
	then
		return $intelligentCooling
	else
		echo "System in unknown mode. STMD: $retSTMD QTMD: $retQTMD"
		exit 4
	fi
}

get_system_performance_mode_15()
{
	echo "\_SB.PCI0.LPC0.EC0.SPMO" > $acpi_call_path
	local retSPMO=$(get_acpi_return_code)
	echo "\_SB.PCI0.LPC0.EC0.FCMO" > $acpi_call_path
	local retFCMO=$(get_acpi_return_code)
	if [ "$retSPMO" = $acpiReturnValue0 ] && [ "$retFCMO" = $acpiReturnValue0 ]
	then
		return $intelligentCooling
	elif [ "$retSPMO" = $acpiReturnValue1 ] && [ "$retFCMO" = $acpiReturnValue1 ]
	then
		return $extremePerformance
	elif [ "$retSPMO" = $acpiReturnValue2 ] && [ "$retFCMO" = $acpiReturnValue2 ]
	then
		return $batterySaving
	else
		echo "System in unknown mode. SPMO: $retSPMO FCMO: $retFCMO"
		exit 5
	fi
}

get_system_performance_mode()
{
	load_acpi_module
	case $system_version in
		14 )	get_system_performance_mode_14
			;;
		15 )	get_system_performance_mode_15
			;;
	esac
	return $?
}

set_system_performance_mode()
{
	local -i arg1=$1
	load_acpi_module
	case $arg1 in
		$extremePerformance )	echo "Activating Extreme Performance mode."
					echo "\_SB.PCI0.LPC0.EC0.VPC0.DYTC 0x0012B001" > $acpi_call_path
					;;
		$batterySaving )	echo "Activating Battery Saving mode."
					echo "\_SB.PCI0.LPC0.EC0.VPC0.DYTC 0x0013B001" > $acpi_call_path
					;;
		$intelligentCooling )	echo "Activating Intelligent Cooling mode."
					echo "\_SB.PCI0.LPC0.EC0.VPC0.DYTC 0x000FB001" > $acpi_call_path
					;;
		* ) 			echo "Invalid Parameter $arg1."
					exit 6
	esac
}

print_system_performance_mode()
{
	get_system_performance_mode
	local -i retSysPerfMode=$?
	case $retSysPerfMode in
		$extremePerformance )	echo "System in Extreme Performance mode."
					;;
		$batterySaving )	echo "System in Battery Saving mode."
					;;
		$intelligentCooling )	echo "System in Intelligent Cooling mode."
					;;
		* )			echo "Invalid Parameter $retSysPerfMode."
					exit 6
	esac
}

get_rapid_charge_mode()
{
	load_acpi_module
	case $system_version in
		14 )	echo "\_SB.PCI0.LPC0.EC0.FCGM" > $acpi_call_path
			;;
		15 )	echo "\_SB.PCI0.LPC0.EC0.QCHO" > $acpi_call_path
			;;
	esac
	local retACPI=$(get_acpi_return_code)
	case "$retACPI" in
		$acpiReturnValue0 )	return 0
					;;
		$acpiReturnValue1 )	return 1
					;;
		* )			echo "${retACPI}"
					exit 7
	esac
}

set_rapid_charge_mode()
{
	local -i arg1=$1
	load_acpi_module
	if [ $arg1 -eq 0 ]
	then 
		echo "\_SB.PCI0.LPC0.EC0.VPC0.SBMC 0x08" > $acpi_call_path
	else
		echo "\_SB.PCI0.LPC0.EC0.VPC0.SBMC 0x07" > $acpi_call_path
	fi
}

toggle_rapid_charge_mode()
{
	get_rapid_charge_mode
	local -i chargeMode=$?
	if [ $chargeMode -eq 0 ]
	then
		echo "Toggling Rapid Charging mode: ON"
		set_rapid_charge_mode 1
	else
		echo "Toggling Rapid Charging mode: OFF"
		set_rapid_charge_mode 0
	fi
}

print_rapid_charge_mode_status()
{
	get_rapid_charge_mode
	local -i retRapidMode=$?
	if [ $retRapidMode -eq 0 ]
	then
		echo "Rapid Charge mode: OFF"
	else
		echo "Rapid Charge mode: ON"
	fi
}	

fix_touchpad()
{
	case $system_version in
		15 )	if [ -e /sys/bus/i2c/drivers/elants_i2c/unbind ]
			then
				echo "i2c-ELAN0001:00" > /sys/bus/i2c/drivers/elants_i2c/unbind
			fi
			echo "i2c-ELAN0001:00" > /sys/bus/i2c/drivers/i2c_hid/bind
			;;
		14 )	echo "Function not implemented!"
			#TODO
			;;
	esac
}

print_info_all()
{
	print_system_version
	print_bios_version
	print_power_state
	print_system_performance_mode
	print_conservation_mode_status
	print_rapid_charge_mode_status
}

usage()
{
	echo "Usage: ideapad.sh [OPTION...]" 
	echo "ideapad.sh changes power modes and charging behaviour of"
	echo "Ideapad 5 14 (14are05) and Ideapad 5 15 (15are05) laptops."
	echo "This script comes without any warrenty - use on your own risk!"
	echo ""
	echo "Examples:"
	echo ""
	echo "  ideapad.sh -a			# Display machine type and current states."
	echo "  ideapad.sh -b -a		# Switch to Battery Saving mode, afterwards"
	echo "				  display current states."
	echo ""
	echo " Informative output:"
	echo ""
	echo "  -a, 	--show-all		Display machine type, performance mode,"
	echo "				battery conservation state and rapid charging state."
	echo "  	--get-conservation	Display state of Battery Conservation."
	echo "  	--get-rapid-charge	Display state of Rapid Charging."
	echo "  -p, 	--get-performance-mode"
	echo "				Display current System Performance mode."
	echo "  -h, -?, --help		Show this help and exit."
	echo "  -v, 	--version		Show version of the script and exit."
	echo ""
	echo " Battery Conservation mode:"
	echo ""
	echo "  -c, 	--conservation		Toggle Battery Conservation on and off."
	echo "  	--conservation-on	Switch Battery Conservation on."
	echo "  	--conservation-off	Switch Battery Conservation off."
	echo ""
	echo " Rapid Charge mode:"
	echo ""
	echo "  -r, 	--rapid-charge		Toggle Rapid Charge on and off."
	echo "  	--rapid-charge-on	Switch Rapid Charge on."
	echo "  	--rapid-charge-off	Switch Rapid Charge off."
	echo ""
	echo " System Performance mode:"
	echo ""
	echo "  -e, 	--extreme-performance"
	echo "				Switch system into Extreme Performance mode."
	echo "  -i, 	--intelligent-cooling"
	echo "				Switch system into Intelligent Cooling mode."
	echo "  -b, 	--battery-saving	Switch system into Battery Saving mode."
	echo ""
	echo " Other options:"
	echo ""
	echo "  -f, 	--fix-touchpad		Execute commands to fix the touchpad of the"
	echo "				Ideapad 5 15 (15are05)."
}

check_root
get_system_version
declare -ri system_version=$?

while [ "$1" != "" ]; do
	case $1 in
		-a | --show-all )		print_info_all
						;;
		-c | --conservation )		toggle_conservation_mode
						;;
		--get-conservation )		print_conservation_mode_status
						;;
		--conservation-on )		set_conservation_mode 1;print_conservation_mode_status
						;;
		--conservation-off )		set_conservation_mode 0;print_conservation_mode_status
						;;
		-r | --rapid-charge )		toggle_rapid_charge_mode
						;;
		--get-rapid-charge )		print_rapid_charge_mode_status
						;;
		--rapid-charge-on )		set_rapid_charge_mode 1;print_rapid_charge_mode_status
						;;
		--rapid-charge-off )		set_rapid_charge_mode 0;print_rapid_charge_mode_status
						;;
		-p | --get-performance-mode )	print_system_performance_mode
						;;
		-e | --extreme-performance )	set_system_performance_mode $extremePerformance
						;;
		-i | --intelligent-cooling )	set_system_performance_mode $intelligentCooling
						;;
		-b | --battery-saving )		set_system_performance_mode $batterySaving
						;;
		-f | --fix-touchpad )		fix_touchpad
						;;
		-h | -\? | --help )		usage
						exit
						;;
		-v | --version )		echo "Script version $scriptVersion"
						exit
						;;		
		* )				usage
						exit 1
	esac
	shift
done

